@@ -2,13 +2,8 @@ import requests
 import time
 from PIL import Image
 import typing
-from io import BytesIO
-
-class RateLimitException(Exception):
-    """
-    An exception class to use whenever cooldown occurs
-    """
-    pass
+from rich.progress import track
+import datetime
 
 class OutOfBoundsException(Exception):
     """
@@ -23,16 +18,45 @@ class Client:
 
     def __init__(self, token):
         self.token = token
-        self.post_limit = 1
-        self.post_timeout = 1
-        self.get_limit = 1
-        self.get_timeout = 1
-        self.gets_limit = 1
-        self.gets_timeout = 1
         self.headers = {"Authorization": f"Bearer {self.token}"}
         self.__http = requests.Session()
         self.base = "https://pixels.pythondiscord.com"
 
+        with self.__http.head(self.base + "/set_pixel", headers=self.headers) as resp:
+            headers = resp.headers
+            try:
+                self.post_limit = headers["requests-remaining"]
+            except KeyError:
+                self.post_limit = 0
+            if self.post_limit < headers["requests-limit"]:
+                self.post_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['requests-reset']))
+            else:
+                self.post_timeout = datetime.datetime.now()
+
+        
+        with self.__http.head(self.base + "/get_pixel", headers=self.headers) as resp:
+            headers = resp.headers
+            try:
+                self.get_limit = headers["requests-remaining"]
+            except KeyError:
+                self.get_limit = 0
+            if self.get_limit < headers["requests-limit"]:
+                self.get_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['requests-reset']))
+            else:
+                self.get_timeout = datetime.datetime.now()
+
+        with self.__http.head(self.base + "/get_pixels", headers=self.headers) as resp:
+            headers = resp.headers
+            try:
+                self.gets_limit = headers["requests-remaining"]
+            except KeyError:
+                self.gets_limit = 0
+            if self.gets_limit < headers["requests-limit"]:
+                self.gets_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['requests-reset']))
+            else:
+                self.gets_timeout = datetime.datetime.now()
+        
+    
     def get_pixel(self, x: int, y: int):
         """
         Returns the hexadecimal color code of the given pixel
@@ -42,12 +66,13 @@ class Client:
         y: int - The y position of the pixel
 
         Returns:
-        int base 16 - The color of the requested pixel
+        int - The color of the requested pixel
         """
-
-        if self.get_limit == 0:
-            print(f"Waiting {self.get_timeout} seconds on /get_pixel rate limit")
-            time.sleep(self.get_timeout + 1)
+        
+        if self.get_limit == 0 and self.get_timeout > datetime.datetime.now():
+            timer = self.get_timeout - datetime.datetime.now()
+            for n in track(range(int(timer.total_seconds()) + 1), "[cyan bold]Awaiting /get_pixel rate limit.."):
+                time.sleep(1)
 
         with self.__http.get(self.base + "/get_size", headers=self.headers) as resp:
             data = resp.json()
@@ -65,13 +90,12 @@ class Client:
             headers = resp.headers
             try:
                 self.get_limit = int(headers["requests-remaining"])
-                self.get_timeout = int(headers["requests-reset"])
+                self.get_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['requests-reset']))
             except KeyError:
                 self.get_limit = 0
-                self.get_timeout = int(headers['cooldown-reset'])
-                raise RateLimitException(f"Rate limited by server but internal values not updated. Please wait {self.get_timeout}")
+                self.get_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['cooldown-reset']))
             return int(f"0x{data['rgb']}", base=16)
-    
+
     def get_canvas(self, scale=1):
         """
         Fetch the entire canvas and returns it as a pillow Image instance. Optionally resize it by a scale factor
@@ -84,22 +108,21 @@ class Client:
         """
         if scale <=0:
             raise TypeError("Scale must be a positive integer")
-        if self.gets_limit == 0:
-            print(f"Waiting {self.gets_timeout} seconds on /get_pixels rate limit")
-            time.sleep(self.gets_timeout+ 1)
+        if self.gets_limit == 0 and self.gets_timeout > datetime.datetime.now():
+            timer = self.gets_timeout - datetime.datetime.now()
+            for n in track(range(int(timer.total_seconds()) + 1), "[cyan bold]Awaiting /get_canvas rate limit.."):
+                time.sleep(1)
         with self.__http.get(self.base + "/get_pixels", headers=self.headers) as resp:
-            stream = bytes(resp.content)
             headers = resp.headers
             try:
                 self.gets_limit = int(headers["requests-remaining"])
-                self.gets_timeout = int(headers["requests-reset"])
+                self.gets_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['requests-reset']))
             except KeyError:
                 self.gets_limit = 0
-                self.gets_timeout = int(headers['cooldown-reset'])
-                raise RateLimitException(f"Rate limited by server but internal values not updated. Please wait {self.gets_timeout}")
+                self.gets_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['cooldown-reset']))
             with self.__http.get(self.base + "/get_size", headers= self.headers) as resps:
                 size = resps.json()
-                im =  Image.frombytes(mode="RGB", size=(size['width'], size['height']), data=stream)
+                im =  Image.frombytes(mode="RGB", size=(size['width'], size['height']), data=resp.content)
                 im = im.resize((im.size[0]*scale, im.size[1]*scale), Image.NEAREST)
                 return im
 
@@ -159,19 +182,18 @@ class Client:
         elif len(data["rgb"]) < 6:
             while len(data["rgb"]) < 6:
                 data["rgb"] = "0" + data["rgb"]
-        if self.post_limit == 0:
-            print(f"Waiting {self.post_timeout} seconds on /set_pixel rate limit")
-            time.sleep(self.post_timeout + 1)
+        if self.post_limit == 0 and self.post_timeout > datetime.datetime.now():
+            timer = self.post_timeout - datetime.datetime.now()
+            for n in track(range(int(timer.total_seconds()) + 1), "[cyan bold]Awaiting /set_pixel rate limit.."):
+                time.sleep(1)
         with self.__http.post(self.base + "/set_pixel", headers=self.headers, json=data) as resp:
             headers = resp.headers
             try:
                 self.post_limit = int(headers["requests-remaining"])
-                self.post_timeout = int(headers["requests-reset"])
+                self.post_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['requests-reset']))
             except KeyError:
                 self.post_limit = 0
-                self.post_timeout = int(headers['cooldown-reset'])
-
-                raise RateLimitException(f"Rate limited by server but internal values not updated. Please wait {self.post_timeout}")
+                self.post_timeout = datetime.datetime.now() + datetime.timedelta(seconds=float(headers['cooldown-reset']))
             return
 
     def get_limits(self):
@@ -245,5 +267,5 @@ class Client:
                 if cur == int(color, base=16):
                     continue
 
-                self.set_pixel(x, y, color)
+                self.set_pixel(x + ox, y + oy, color)
 
